/*
 *  TOPPERS/FMP Kernel
 *      Toyohashi Open Platform for Embedded Real-Time Systems/
 *      Advanced Standard Profile Kernel
 * 
 *  Copyright (C) 2000-2003 by Embedded and Real-Time Systems Laboratory
 *                              Toyohashi Univ. of Technology, JAPAN
 *  Copyright (C) 2005-2014 by Embedded and Real-Time Systems Laboratory
 *              Graduate School of Information Science, Nagoya Univ., JAPAN
 *  Copyright (C) 2016-2022 by TOPPERS PROJECT Educational Working Group.
 * 
 *  上記著作権者は，以下の(1)?(4)の条件を満たす場合に限り，本ソフトウェ
 *  ア（本ソフトウェアを改変したものを含む．以下同じ）を使用・複製・改
 *  変・再配布（以下，利用と呼ぶ）することを無償で許諾する．
 *  (1) 本ソフトウェアをソースコードの形で利用する場合には，上記の著作
 *      権表示，この利用条件および下記の無保証規定が，そのままの形でソー
 *      スコード中に含まれていること．
 *  (2) 本ソフトウェアを，ライブラリ形式など，他のソフトウェア開発に使
 *      用できる形で再配布する場合には，再配布に伴うドキュメント（利用
 *      者マニュアルなど）に，上記の著作権表示，この利用条件および下記
 *      の無保証規定を掲載すること．
 *  (3) 本ソフトウェアを，機器に組み込むなど，他のソフトウェア開発に使
 *      用できない形で再配布する場合には，次のいずれかの条件を満たすこ
 *      と．
 *    (a) 再配布に伴うドキュメント（利用者マニュアルなど）に，上記の著
 *        作権表示，この利用条件および下記の無保証規定を掲載すること．
 *    (b) 再配布の形態を，別に定める方法によって，TOPPERSプロジェクトに
 *        報告すること．
 *  (4) 本ソフトウェアの利用により直接的または間接的に生じるいかなる損
 *      害からも，上記著作権者およびTOPPERSプロジェクトを免責すること．
 *      また，本ソフトウェアのユーザまたはエンドユーザからのいかなる理
 *      由に基づく請求からも，上記著作権者およびTOPPERSプロジェクトを
 *      免責すること．
 * 
 *  本ソフトウェアは，無保証で提供されているものである．上記著作権者お
 *  よびTOPPERSプロジェクトは，本ソフトウェアに関して，特定の使用目的
 *  に対する適合性も含めて，いかなる保証も行わない．また，本ソフトウェ
 *  アの利用により直接的または間接的に生じたいかなる損害に関しても，そ
 *  の責任を負わない．
 * 
 *  @(#) $Id: core_test.h 292 698 2022-09-25 16:45:35Z roi $
 */

/*
 *		テストプログラムのチップ依存定義（ARM-M用）
 */

#ifndef TOPPERS_CORE_TEST_H
#define TOPPERS_CORE_TEST_H

#if __TARGET_ARCH_THUMB == 4
#define CPUEXC1 (0x10000 | 6) /* Usage fault */
#define CPUEXC2 (0x20000 | 6)
#if TNUM_PRCID >= 3
#define CPUEXC3 (0x30000 | 6)
#endif
#if TNUM_PRCID >= 4
#define CPUEXC4 (0x40000 | 6)
#endif
#define RAISE_CPU_EXCEPTION Asm("mcr p15, 0, r1, c2, c0, 0");
#define CANNOT_RETURN_CPUEXC
#else /* __TARGET_ARCH_THUMB == 3 */
#define CPUEXC1 (0x10000 | 3) /* Hard fault */
#define CPUEXC2 (0x20000 | 3)
#if TNUM_PRCID >= 3
#define CPUEXC3 (0x30000 | 3)
#endif
#if TNUM_PRCID >= 4
#define CPUEXC4 (0x40000 | 3)
#endif
#define RAISE_CPU_EXCEPTION *(volatile int*)(0x1) = 0x11;
#define CANNOT_RETURN_CPUEXC
#endif /* __TARGET_ARCH_THUMB == 4 */

#endif /* TOPPERS_CORE_TEST_H */
