/*
 *  TOPPERS/FMP Kernel
 *      Toyohashi Open Platform for Embedded Real-Time Systems/
 *      Flexible MultiProcessor Kernel
 *
 *  Copyright (C) 2009-2020 by Embedded and Real-Time Systems Laboratory
 *              Graduate School of Information Science, Nagoya Univ., JAPAN
 *  Copyright (C) 2016-2022 by TOPPERS PROJECT Educational Working Group.
 *
 *  上記著作権者は，以下の(1)～(4)の条件を満たす場合に限り，本ソフトウェ
 *  ア（本ソフトウェアを改変したものを含む．以下同じ）を使用・複製・改
 *  変・再配布（以下，利用と呼ぶ）することを無償で許諾する．
 *  (1) 本ソフトウェアをソースコードの形で利用する場合には，上記の著作
 *      権表示，この利用条件および下記の無保証規定が，そのままの形でソー
 *      スコード中に含まれていること．
 *  (2) 本ソフトウェアを，ライブラリ形式など，他のソフトウェア開発に使
 *      用できる形で再配布する場合には，再配布に伴うドキュメント（利用
 *      者マニュアルなど）に，上記の著作権表示，この利用条件および下記
 *      の無保証規定を掲載すること．
 *  (3) 本ソフトウェアを，機器に組み込むなど，他のソフトウェア開発に使
 *      用できない形で再配布する場合には，次のいずれかの条件を満たすこ
 *      と．
 *    (a) 再配布に伴うドキュメント（利用者マニュアルなど）に，上記の著
 *        作権表示，この利用条件および下記の無保証規定を掲載すること．
 *    (b) 再配布の形態を，別に定める方法によって，TOPPERSプロジェクトに
 *        報告すること．
 *  (4) 本ソフトウェアの利用により直接的または間接的に生じるいかなる損
 *      害からも，上記著作権者およびTOPPERSプロジェクトを免責すること．
 *      また，本ソフトウェアのユーザまたはエンドユーザからのいかなる理
 *      由に基づく請求からも，上記著作権者およびTOPPERSプロジェクトを
 *      免責すること．
 *
 *  本ソフトウェアは，無保証で提供されているものである．上記著作権者お
 *  よびTOPPERSプロジェクトは，本ソフトウェアに関して，特定の使用目的
 *  に対する適合性も含めて，いかなる保証も行わない．また，本ソフトウェ
 *  アの利用により直接的または間接的に生じたいかなる損害に関しても，そ
 *  の責任を負わない．
 *
 *  @(#) $Id: chip_ipi.c 289 2022-12-02 15:12:22Z roi $
 */

/*
 *  プロセッサ間割込みのチップ依存部（RP2040用）
 */

#include "kernel_impl.h"
#include "time_event.h"
#include "chip_ipi.h"

static FP (*ipi_callback[TNUM_PRCID])(uint32_t);

/*
 *  IPIの初期化を行う
 */
void
chip_ipi_initialize(intptr_t exinf)
{
    sil_wrw_mem((uint32_t *)(TADR_SIO_FIFO_ST),
                SIO_FIFO_ST_ROE_BIT | SIO_FIFO_ST_WOF_BIT);
    chip_ipi_clear_int(INTNO_IPI_PRC1);
    chip_ipi_clear_int(INTNO_IPI_PRC2);
	ipi_callback[x_prc_index()] = NULL;
}

/*
 *  IPIのイベント送信を行う
 */
ER
send_ipi_event(uint32_t event)
{
	if(event >= IPINO_DISPATCH)
		return E_PAR;
	request_dispatch_prc(event);
	return E_OK;
}

/*
 *  IPIのイベント受信コールバック関数を設定する
 */
void
set_ipi_callback(void *func)
{
	ipi_callback[x_prc_index()] = func;
}

/*
 *  IPI割込みハンドラ
 */
void
_kernel_ipi_irq_handler(void)
{
	uint32_t event = sil_rew_mem((uint32_t *)(TADR_SIO_FIFO_RD));
	uint32_t idx = x_prc_index();

	if(event == IPINO_DISPATCH)
		ipi_handler();
	else{
		if(ipi_callback[idx] != NULL)
			ipi_callback[idx](event);
	}
}
