/*
 *  TOPPERS Software
 *      Toyohashi Open Platform for Embedded Real-Time Systems
 *
 *  Copyright (C) 2007,2011,2015 by Embedded and Real-Time Systems Laboratory
 *              Graduate School of Information Science, Nagoya Univ., JAPAN
 *  Copyright (C) 2016-2022 by TOPPERS PROJECT Educational Working Group.
 *
 *  上記著作権者は，以下の(1)～(4)の条件を満たす場合に限り，本ソフトウェ
 *  ア（本ソフトウェアを改変したものを含む．以下同じ）を使用・複製・改
 *  変・再配布（以下，利用と呼ぶ）することを無償で許諾する．
 *  (1) 本ソフトウェアをソースコードの形で利用する場合には，上記の著作
 *      権表示，この利用条件および下記の無保証規定が，そのままの形でソー
 *      スコード中に含まれていること．
 *  (2) 本ソフトウェアを，ライブラリ形式など，他のソフトウェア開発に使
 *      用できる形で再配布する場合には，再配布に伴うドキュメント（利用
 *      者マニュアルなど）に，上記の著作権表示，この利用条件および下記
 *      の無保証規定を掲載すること．
 *  (3) 本ソフトウェアを，機器に組み込むなど，他のソフトウェア開発に使
 *      用できない形で再配布する場合には，次のいずれかの条件を満たすこ
 *      と．
 *    (a) 再配布に伴うドキュメント（利用者マニュアルなど）に，上記の著
 *        作権表示，この利用条件および下記の無保証規定を掲載すること．
 *    (b) 再配布の形態を，別に定める方法によって，TOPPERSプロジェクトに
 *        報告すること．
 *  (4) 本ソフトウェアの利用により直接的または間接的に生じるいかなる損
 *      害からも，上記著作権者およびTOPPERSプロジェクトを免責すること．
 *      また，本ソフトウェアのユーザまたはエンドユーザからのいかなる理
 *      由に基づく請求からも，上記著作権者およびTOPPERSプロジェクトを
 *      免責すること．
 *
 *  本ソフトウェアは，無保証で提供されているものである．上記著作権者お
 *  よびTOPPERSプロジェクトは，本ソフトウェアに関して，特定の使用目的
 *  に対する適合性も含めて，いかなる保証も行わない．また，本ソフトウェ
 *  アの利用により直接的または間接的に生じたいかなる損害に関しても，そ
 *  の責任を負わない．
 *
 *  @(#) $Id: chip_ipi.c 289 2022-12-11 06:51:56Z roi $
 */

#ifndef RP2040_SIO_H
#define RP2040_SIO_H

/*
 *  マルチプロセッサ制御使用するSIOの定義
 */
#define TADR_SIO_CPUID          0xD0000000
#define TADR_SIO_FIFO_ST        0xD0000050	/* (R)  Status register for inter-core FIFOs (mailboxes) */
  #define SIO_FIFO_ST_VLD_BIT   0x00000001	/* VLD (0): Value is 1 if this core's RX FIFO is not empty (i */
  #define SIO_FIFO_ST_RDY_BIT   0x00000002	/* RDY (1): Value is 1 if this core's TX FIFO is not full (i */
  #define SIO_FIFO_ST_WOF_BIT   0x00000004	/* WOF (0): Sticky flag indicating the TX FIFO was written when full */
  #define SIO_FIFO_ST_ROE_BIT   0x00000008	/* ROE (0): Sticky flag indicating the RX FIFO was read when empty */
#define TADR_SIO_FIFO_WR        0xD0000054	/* (W)  Write access to this core's TX FIFO */
#define TADR_SIO_FIFO_RD        0xD0000058	/* (R)  Read access to this core's RX FIFO */
#define TADR_SIO_SPNLOCK        0xD0000100

#ifndef TOPPERS_MACRO_ONLY

/*
 *  プロセッサINDEX（0オリジン）の取得
 */
Inline uint32_t
x_prc_index(void)
{
    return *((volatile uint32_t *)TADR_SIO_CPUID);
}

#endif /* TOPPERS_MACRO_ONLY */
#endif /* RP2040_SIO_H */
