/*
 *  TOPPERS/FMP Kernel
 *      Toyohashi Open Platform for Embedded Real-Time Systems/
 *      Advanced Standard Profile Kernel
 * 
 *  Copyright (C) 2008-2011 by Embedded and Real-Time Systems Laboratory
 *              Graduate School of Information Science, Nagoya Univ., JAPAN
 *  Copyright (C) 2015-2016 by 3rd Designing Center
 *              Imageing System Development Division RICOH COMPANY, LTD.
 *  Copyright (C) 2016-2022 by TOPPERS PROJECT Educational Working Group.
 * 
 *  上記著作権者は，以下の(1)～(4)の条件を満たす場合に限り，本ソフトウェ
 *  ア（本ソフトウェアを改変したものを含む．以下同じ）を使用・複製・改
 *  変・再配布（以下，利用と呼ぶ）することを無償で許諾する．
 *  (1) 本ソフトウェアをソースコードの形で利用する場合には，上記の著作
 *      権表示，この利用条件および下記の無保証規定が，そのままの形でソー
 *      スコード中に含まれていること．
 *  (2) 本ソフトウェアを，ライブラリ形式など，他のソフトウェア開発に使
 *      用できる形で再配布する場合には，再配布に伴うドキュメント（利用
 *      者マニュアルなど）に，上記の著作権表示，この利用条件および下記
 *      の無保証規定を掲載すること．
 *  (3) 本ソフトウェアを，機器に組み込むなど，他のソフトウェア開発に使
 *      用できない形で再配布する場合には，次のいずれかの条件を満たすこ
 *      と．
 *    (a) 再配布に伴うドキュメント（利用者マニュアルなど）に，上記の著
 *        作権表示，この利用条件および下記の無保証規定を掲載すること．
 *    (b) 再配布の形態を，別に定める方法によって，TOPPERSプロジェクトに
 *        報告すること．
 *  (4) 本ソフトウェアの利用により直接的または間接的に生じるいかなる損
 *      害からも，上記著作権者およびTOPPERSプロジェクトを免責すること．
 *      また，本ソフトウェアのユーザまたはエンドユーザからのいかなる理
 *      由に基づく請求からも，上記著作権者およびTOPPERSプロジェクトを
 *      免責すること．
 * 
 *  本ソフトウェアは，無保証で提供されているものである．上記著作権者お
 *  よびTOPPERSプロジェクトは，本ソフトウェアに関して，特定の使用目的
 *  に対する適合性も含めて，いかなる保証も行わない．また，本ソフトウェ
 *  アの利用により直接的または間接的に生じたいかなる損害に関しても，そ
 *  の責任を負わない．
 * 
 *  @(#) $Id: target_config.h 698 2022-12-10 17:08:35Z roi $
 */

/*
 *  チップ依存モジュール（raspberrypi_pico用）
 *
 *  カーネルのターゲット依存部のインクルードファイル．kernel_impl.hのター
 *  ゲット依存部の位置付けとなる．
 */

#ifndef TOPPERS_TARGET_CONFIG_H
#define TOPPERS_TARGET_CONFIG_H

/*
 *  割込み番号の最大値
 */
#define TMAX_INTNO              (16 + 26)

/*
 *  割込みベクター定義
 */
#define IRQ_VECTOR_TIMER0       (16 + 0)	/* TIMER_IRQ_0 */
#define IRQ_VECTOR_TIMER1       (16 + 1)	/* TIMER_IRQ_1 */
#define IRQ_VECTOR_TIMER2       (16 + 2)	/* TIMER_IRQ_2 */
#define IRQ_VECTOR_TIMER3       (16 + 3)	/* TIMER_IRQ_3 */
#define IRQ_VECTOR_PWM_WRAP     (16 + 4)	/* PWM_IRQ_WRAP */
#define IRQ_VECTOR_USBCTRL      (16 + 5)	/* USBCTRL_IRQ */
#define IRQ_VECTOR_XIP          (16 + 6)	/* XIP_IRQ */
#define IRQ_VECTOR_PIO0_0       (16 + 7)	/* PIO0_IRQ_0 */
#define IRQ_VECTOR_PIO0_1       (16 + 8)	/* PIO0_IRQ_1 */
#define IRQ_VECTOR_PIO1_0       (16 + 9)	/* PIO1_IRQ_0 */
#define IRQ_VECTOR_PIO1_1       (16 + 10)	/* PIO1_IRQ_1 */
#define IRQ_VECTOR_DMA0         (16 + 11)	/* DMA_IRQ_0  */
#define IRQ_VECTOR_DMA1         (16 + 12)	/* DMA_IRQ_1 */
#define IRQ_VECTOR_IO_BANK0     (16 + 13)	/* IO_IRQ_BANK0  */
#define IRQ_VECTOR_IO_QSPI      (16 + 14)	/* IO_IRQ_QSPI */
#define IRQ_VECTOR_SIO_PROC0    (16 + 15)	/* SIO_IRQ_PROC0 */
#define IRQ_VECTOR_SIO_PROC1    (16 + 16)	/* SIO_IRQ_PROC1 */
#define IRQ_VECTOR_CLOCKS       (16 + 17)	/* CLOCKS_IRQ */
#define IRQ_VECTOR_SPI0         (16 + 18)	/* SPI0_IRQ */
#define IRQ_VECTOR_SPI1         (16 + 19)	/* SPI1_IRQ */
#define IRQ_VECTOR_UART0        (16 + 20)	/* UART0_IRQ */
#define IRQ_VECTOR_UART1        (16 + 21)	/* UART1_IRQ */
#define IRQ_VECTOR_ADC_FIFO     (16 + 22)	/* ADC0_IRQ_FIFO */
#define IRQ_VECTOR_I2C0         (16 + 23)	/* I2C0_IRQ  */
#define IRQ_VECTOR_I2C1         (16 + 24)	/* I2C1_IRQ */
#define IRQ_VECTOR_RTC          (16 + 25)	/* RTC Interrupt  */

/*
 *  微少時間待ちのための定義（本来はSILのターゲット依存部）
 *  133-125MHz用に設定、クロックを下げた場合、時間は増大する．
 */
#define SIL_DLY_TIM1    40
#define SIL_DLY_TIM2    32

#ifndef TOPPERS_MACRO_ONLY

/*
 *  str_ker()関数呼び出し前の、マスタープロセッサ初期化
 */
extern void target_mprc_initialize(void);

/*
 *  ターゲットシステム依存の初期化
 */
extern void	target_initialize(void);

/*
 *  ターゲットシステムの終了
 *
 *  システムを終了する時に使う．
 */
extern void	target_exit(void) NoReturn;

#endif /* TOPPERS_MACRO_ONLY */

/*
 *  トレースログに関する設定
 */
#ifdef TOPPERS_ENABLE_TRACE
#include "logtrace/trace_config.h"
#endif /* TOPPERS_ENABLE_TRACE */

/*
 *  Spinlock implementation to use
 *      - NATIVE_SPN: no contention with other syscalls
 *      - EMULATE_SPN: contention with other syscalls
 */
#define TTYPE_SPN NATIVE_SPN
//#define TTYPE_SPN EMULATE_SPN

/*
 *  Default size of non-task stack for storing context
 */
#define DEFAULT_PRC1_ISTKSZ      (1024 * 4)   /* 4KB */
#define DEFAULT_PRC2_ISTKSZ      (1024 * 4)   /* 4KB */

#ifndef TOPPERS_MACRO_ONLY

/*
 *  起動システムクロックの取得
 */
extern uint32_t sysclock_init_value(void);

/*
 *  システムクロックの変更
 */
extern ER sysclock_change(uint8_t range);

#endif /* TOPPERS_MACRO_ONLY */


/*
 *  チップ依存モジュール（rp2040用）
 */
#include "arm_m_gcc/rp2040/chip_config.h"

#endif /* TOPPERS_TARGET_CONFIG_H */
